{"cells": [{"cell_type": "markdown", "id": "d9d5e1c2", "metadata": {"editable": true, "slideshow": {"slide_type": ""}, "tags": []}, "source": ["## Recovering Correlation Energy: B atom"]}, {"cell_type": "markdown", "id": "4e11f759", "metadata": {"editable": true, "slideshow": {"slide_type": ""}, "tags": []}, "source": [":::{panels} \n", ":column: col-12\n", "Based on **Basis sets**\n", " Tricia D. Shepherd [![Orcid](../../images/orcid.png)](https://orcid.org/0000-0001-6512-8951), Ryan C. Fortenberry, Matthewy Kennedy, C. David Sheril, 2008-2019, The Psi4Education Developers &nbsp;&nbsp; License BSD-3\n", ":::"]}, {"cell_type": "markdown", "id": "cb2bb7de", "metadata": {}, "source": ["At the Hartree Fock level of theory, each electron experiences an average potential field of all the other electrons. In essence, it is a \"mean field\" approach that neglects individual electron-electron interactions or \"electron correlation\". Thus, we define the difference between the self-consistent field energy and the exact energy as the correlation energy. Two fundamentally different approaches to account for electron correlation effects are available by selecting a Correlation method: Moller Plesset (MP) Perturbation theory and Coupled Cluster (CC) theory. "]}, {"cell_type": "markdown", "id": "557c19ec-1191-432b-8394-7b57da786eea", "metadata": {"editable": true, "slideshow": {"slide_type": ""}, "tags": []}, "source": [":::{admonition} Exercise 1\n", ":class: exercise \n", "\n", "* Calculate the HF energy for Boron using the 6-311+G** basis set, then determine the value of the correlation energy for boron assuming an \"experimental\" energy of -24.608 Hartrees ([Schaefer III, Henry F., and Frank E. Harris. \"Electronic Structure of Atomic Boron.\" Physical Review 167.1 (1968): 67](https://journals.aps.org/pr/abstract/10.1103/PhysRev.167.67)).  \n", "* Using the same basis set, perform an energy calculation with CISD and full CI. \n", "* Using the same basis set, perform an energy calculation with MP2, MP3 and MP5. \n", "* Using the same basis set, perform an energy calculation with CCSD and CCSD(T).\n", "\n", "Determine the percentage of the correlation energy recovered for HF, MP2, MP5, CCSD, CCSD(T). \n", "\n", "Compare the performances of the different methods. In particular, you will see that some of the methods seem to overestimate the correlation recovered. How is it possible? (*Hint:* Think both about the theoretical aspects of the methods and *how* we are computing the recovered correlation energy).\n", ":::"]}, {"cell_type": "markdown", "id": "41dd660e-632d-47a0-a40e-1151375455c0", "metadata": {"editable": true, "slideshow": {"slide_type": ""}, "tags": []}, "source": [":::{admonition} CCSD energy/mp2 energy\n", ":class: tip\n", "You may recover the CCSD energy from the CCSD(T) calculation, but you will have to look at the output file or by calculating both. In any case, the calculations for a single atom are quick.  \n", "\n", "The same is true for the MP2 energy which can be extracted from the MP5 calculation.\n", "MP5 will require the use of the following options: `psi4.set_options({\"reference\" :\"rohf\", \"qc_module\":\"detci\"})` as Psi4 does not have MP5 implemented using a RHF reference. \n", ":::"]}, {"cell_type": "code", "execution_count": 1, "id": "b59cce45", "metadata": {"editable": true, "execution": {"iopub.execute_input": "2024-11-01T12:55:09.457146Z", "iopub.status.busy": "2024-11-01T12:55:09.456606Z", "iopub.status.idle": "2024-11-01T12:55:11.461535Z", "shell.execute_reply": "2024-11-01T12:55:11.459981Z", "shell.execute_reply.started": "2024-11-01T12:55:09.457102Z"}, "slideshow": {"slide_type": ""}, "tags": []}, "outputs": [], "source": ["import psi4\n", "import py3Dmol\n", "import pandas as pd\n", "import matplotlib.pyplot as plt\n", "\n", "import sys\n", "sys.path.append(\"..\")\n", "from helpers import *"]}, {"cell_type": "code", "execution_count": 2, "id": "71c0d150", "metadata": {"editable": true, "execution": {"iopub.execute_input": "2024-11-01T12:55:11.463880Z", "iopub.status.busy": "2024-11-01T12:55:11.463346Z", "iopub.status.idle": "2024-11-01T12:55:11.479400Z", "shell.execute_reply": "2024-11-01T12:55:11.478241Z", "shell.execute_reply.started": "2024-11-01T12:55:11.463848Z"}, "slideshow": {"slide_type": ""}, "tags": []}, "outputs": [{"name": "stdout", "output_type": "stream", "text": ["  Threads set to 2 by Python driver.\n"]}, {"data": {"text/plain": ["2000000000"]}, "execution_count": 2, "metadata": {}, "output_type": "execute_result"}, {"name": "stdout", "output_type": "stream", "text": ["\n", "  Memory set to   1.863 GiB by Python driver.\n"]}], "source": ["psi4.set_num_threads(2)\n", "psi4.set_memory('2 GB')"]}, {"cell_type": "code", "execution_count": null, "id": "5951bdad", "metadata": {"editable": true, "slideshow": {"slide_type": ""}, "tags": []}, "outputs": [], "source": ["psi4.core.clean_options()\n", "\n", "# define a single boron atom and set the correct multiplicity\n", "boron_geometry = psi4.geometry(\"\"\"\"\"\")\n", "\n", "psi4.set_options({'reference' : \"UHF\"})\n", "\n", "# Perform single-point calculation with requested method/basis \n", "B_energy_hf  = 0"]}, {"cell_type": "code", "execution_count": 4, "id": "3f803251-25c8-4383-ae4c-1541e7cb4e44", "metadata": {"editable": true, "execution": {"iopub.execute_input": "2024-11-01T12:55:14.770537Z", "iopub.status.busy": "2024-11-01T12:55:14.770009Z", "iopub.status.idle": "2024-11-01T12:55:14.782424Z", "shell.execute_reply": "2024-11-01T12:55:14.780033Z", "shell.execute_reply.started": "2024-11-01T12:55:14.770508Z"}, "slideshow": {"slide_type": ""}, "tags": []}, "outputs": [{"data": {"application/3dmoljs_load.v0": "<div id=\"3dmolviewer_173046571477176\"  style=\"position: relative; width: 400px; height: 400px;\">\n        <p id=\"3dmolwarning_173046571477176\" style=\"background-color:#ffcccc;color:black\">3Dmol.js failed to load for some reason.  Please check your browser console for error messages.<br></p>\n        </div>\n<script>\n\nvar loadScriptAsync = function(uri){\n  return new Promise((resolve, reject) => {\n    //this is to ignore the existence of requirejs amd\n    var savedexports, savedmodule;\n    if (typeof exports !== 'undefined') savedexports = exports;\n    else exports = {}\n    if (typeof module !== 'undefined') savedmodule = module;\n    else module = {}\n\n    var tag = document.createElement('script');\n    tag.src = uri;\n    tag.async = true;\n    tag.onload = () => {\n        exports = savedexports;\n        module = savedmodule;\n        resolve();\n    };\n  var firstScriptTag = document.getElementsByTagName('script')[0];\n  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n});\n};\n\nif(typeof $3Dmolpromise === 'undefined') {\n$3Dmolpromise = null;\n  $3Dmolpromise = loadScriptAsync('https://cdnjs.cloudflare.com/ajax/libs/3Dmol/2.1.0/3Dmol-min.js');\n}\n\nvar viewer_173046571477176 = null;\nvar warn = document.getElementById(\"3dmolwarning_173046571477176\");\nif(warn) {\n    warn.parentNode.removeChild(warn);\n}\n$3Dmolpromise.then(function() {\nviewer_173046571477176 = $3Dmol.createViewer(document.getElementById(\"3dmolviewer_173046571477176\"),{backgroundColor:\"white\"});\nviewer_173046571477176.zoomTo();\n\tviewer_173046571477176.addModel(\"1\\n\\n B    0.000000000000    0.000000000000    0.000000000000\\n\",\"xyz\");\n\tviewer_173046571477176.setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n\tviewer_173046571477176.zoomTo();\nviewer_173046571477176.render();\n});\n</script>", "text/html": ["<div id=\"3dmolviewer_173046571477176\"  style=\"position: relative; width: 400px; height: 400px;\">\n", "        <p id=\"3dmolwarning_173046571477176\" style=\"background-color:#ffcccc;color:black\">3Dmol.js failed to load for some reason.  Please check your browser console for error messages.<br></p>\n", "        </div>\n", "<script>\n", "\n", "var loadScriptAsync = function(uri){\n", "  return new Promise((resolve, reject) => {\n", "    //this is to ignore the existence of requirejs amd\n", "    var savedexports, savedmodule;\n", "    if (typeof exports !== 'undefined') savedexports = exports;\n", "    else exports = {}\n", "    if (typeof module !== 'undefined') savedmodule = module;\n", "    else module = {}\n", "\n", "    var tag = document.createElement('script');\n", "    tag.src = uri;\n", "    tag.async = true;\n", "    tag.onload = () => {\n", "        exports = savedexports;\n", "        module = savedmodule;\n", "        resolve();\n", "    };\n", "  var firstScriptTag = document.getElementsByTagName('script')[0];\n", "  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n", "});\n", "};\n", "\n", "if(typeof $3Dmolpromise === 'undefined') {\n", "$3Dmolpromise = null;\n", "  $3Dmolpromise = loadScriptAsync('https://cdnjs.cloudflare.com/ajax/libs/3Dmol/2.1.0/3Dmol-min.js');\n", "}\n", "\n", "var viewer_173046571477176 = null;\n", "var warn = document.getElementById(\"3dmolwarning_173046571477176\");\n", "if(warn) {\n", "    warn.parentNode.removeChild(warn);\n", "}\n", "$3Dmolpromise.then(function() {\n", "viewer_173046571477176 = $3Dmol.createViewer(document.getElementById(\"3dmolviewer_173046571477176\"),{backgroundColor:\"white\"});\n", "viewer_173046571477176.zoomTo();\n", "\tviewer_173046571477176.addModel(\"1\\n\\n B    0.000000000000    0.000000000000    0.000000000000\\n\",\"xyz\");\n", "\tviewer_173046571477176.setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n", "\tviewer_173046571477176.zoomTo();\n", "viewer_173046571477176.render();\n", "});\n", "</script>"]}, "metadata": {}, "output_type": "display_data"}], "source": ["drawXYZ(boron_geometry)"]}, {"cell_type": "code", "execution_count": null, "id": "6fe4741a", "metadata": {"editable": true, "slideshow": {"slide_type": ""}, "tags": []}, "outputs": [], "source": ["#Experimental energy of Boron\n", "B_exact_e = -24.608\n", "\n", "\n", "correlation = None # Calculate the total amount of correlation energy that should be recovered using the B_exact_e\n", "\n", "print(F\"\\n Electron correlation: {correlation} Hartrees\")"]}, {"cell_type": "code", "execution_count": 8, "id": "0c659cbd", "metadata": {"editable": true, "execution": {"iopub.execute_input": "2024-11-01T09:46:46.509995Z", "iopub.status.busy": "2024-11-01T09:46:46.509002Z", "iopub.status.idle": "2024-11-01T09:46:46.523479Z", "shell.execute_reply": "2024-11-01T09:46:46.521852Z", "shell.execute_reply.started": "2024-11-01T09:46:46.509923Z"}, "slideshow": {"slide_type": ""}, "tags": []}, "outputs": [], "source": ["# calculate energy for other methods\n", "psi4.core.clean_options()\n", "psi4.core.clean()\n", "psi4.set_options({'reference' : \"UHF\"})\n", "#MP2\n", "e_mp2 = None\n", "#MP3\n", "e_mp3 = None\n", "#CCSD\n", "e_ccsd = None\n", "#CCSD(T)\n", "e_ccsd_t = None\n", "#MP5\n", "psi4.core.clean_options()\n", "psi4.core.clean()\n", "psi4.set_options({\"reference\" :\"rohf\", \n", "                   \"qc_module\":\"detci\"})\n", "e_mp5 = None\n", "#CISD\n", "e_cisd = None\n", "#FCI\n", "e_fci = None"]}, {"cell_type": "code", "execution_count": 12, "id": "4e1989e7", "metadata": {"editable": true, "execution": {"iopub.execute_input": "2024-11-01T09:57:10.550421Z", "iopub.status.busy": "2024-11-01T09:57:10.549104Z", "iopub.status.idle": "2024-11-01T09:57:10.564570Z", "shell.execute_reply": "2024-11-01T09:57:10.562356Z", "shell.execute_reply.started": "2024-11-01T09:57:10.550346Z"}, "slideshow": {"slide_type": ""}, "tags": []}, "outputs": [], "source": ["correlations = {}\n", "correlations['hf']= 0\n", "correlations['mp2']    = None # calculate the % of correlation energy recovered here\n", "correlations['mp3']    = None # calculate the % of correlation energy recovered here\n", "correlations['mp5']    = None # calculate the % of correlation energy recovered here\n", "correlations['ccsd']   = None # calculate the % of correlation energy recovered here\n", "correlations['ccsd(t)'] = None # calculate the % of correlation energy recovered here\n", "correlations['cisd'] = None # calculate the % of correlation energy recovered here\n", "correlations['fci'] = None # calculate the % of correlation energy recovered here"]}, {"cell_type": "code", "execution_count": null, "id": "b1b0a012", "metadata": {"editable": true, "slideshow": {"slide_type": ""}, "tags": []}, "outputs": [], "source": ["pd.DataFrame.from_dict(correlations,orient='index',columns=['% correlation recovered'])"]}, {"cell_type": "code", "execution_count": null, "id": "1e0696d7-3aea-479f-a687-10f85730a3c4", "metadata": {"editable": true, "slideshow": {"slide_type": ""}, "tags": []}, "outputs": [], "source": []}], "metadata": {"celltoolbar": "Tags", "kernelspec": {"display_name": "Computational Chemistry", "language": "python", "name": "compchem"}, "language_info": {"codemirror_mode": {"name": "ipython", "version": 3}, "file_extension": ".py", "mimetype": "text/x-python", "name": "python", "nbconvert_exporter": "python", "pygments_lexer": "ipython3", "version": "3.12.1"}, "unianalytics_cell_mapping": [["d9d5e1c2", "d9d5e1c2"], ["4e11f759", "4e11f759"], ["cb2bb7de", "cb2bb7de"], ["557c19ec-1191-432b-8394-7b57da786eea", "557c19ec-1191-432b-8394-7b57da786eea"], ["c0eba618-eba8-4c69-81c9-3ec747f5e30e", "c0eba618-eba8-4c69-81c9-3ec747f5e30e"], ["41dd660e-632d-47a0-a40e-1151375455c0", "41dd660e-632d-47a0-a40e-1151375455c0"], ["b59cce45", "b59cce45"], ["71c0d150", "71c0d150"], ["5951bdad", "5951bdad"], ["9f03bdff", "9f03bdff"], ["3f803251-25c8-4383-ae4c-1541e7cb4e44", "3f803251-25c8-4383-ae4c-1541e7cb4e44"], ["6fe4741a", "6fe4741a"], ["ef436fdf", "ef436fdf"], ["0c659cbd", "0c659cbd"], ["649e2cbf", "649e2cbf"], ["4e1989e7", "4e1989e7"], ["c92b60fa", "c92b60fa"], ["b1b0a012", "b1b0a012"], ["2aa742a1", "2aa742a1"], ["1e0696d7-3aea-479f-a687-10f85730a3c4", "2aa742a1"]], "unianalytics_notebook_id": "6ce87eb5-4c59-4ea0-a264-b3fb35e21999"}, "nbformat": 4, "nbformat_minor": 5}